unit kemo_usb;
{
  This unit is an interface to the KEMO module M162 (and maybe other KEMO modules in the future.
  Version 1.00
  Author: Nils Wedel, Kemo Electronic GmbH 2013

  everything you need is the class TKemoUSB (at line 37)
}
interface
uses windows,sysutils;

type
  PHIDP_CAPS = ^HIDP_CAPS;
  HIDP_CAPS = packed record
    Usage:                      WORD;
    UsagePage:                  WORD;
    InputReportByteLength:      WORD;
    OutputReportByteLength:     WORD;
    FeatureReportByteLength:    WORD;
    Reserved: array[0..16]of    WORD;

    NumberLinkCollectionNodes:  WORD;

    NumberInputButtonCaps:      WORD;
    NumberInputValueCaps:       WORD;
    NumberInputDataIndices:     WORD;

    NumberOutputButtonCaps:     WORD;
    NumberOutputValueCaps:      WORD;
    NumberOutputDataIndices:    WORD;

    NumberFeatureButtonCaps:    WORD;
    NumberFeatureValueCaps:     WORD;
    NumberFeatureDataIndices:   WORD;
  end;
//______________________________________________________________________________________________________________________
  TKemoUSB = class(TObject)
  private
    hWriteFileEvent,
    hUSB: cardinal;
    usbwHandleInUse: boolean;
    UsbDeviceCaps: HIDP_CAPS;
    function WriteRaw(cmd,var1,var2: byte; var3: byte=$ff; var4: byte=$ff; var5: byte=$ff): boolean;
  public
    function Connected: boolean;
    function SetState(state: byte):boolean;
    function Connect: boolean;
    procedure Disconnect;
    constructor Create;
    destructor Destroy;override;
  end;
//______________________________________________________________________________________________________________________

  PSP_DEVINFO_DATA = Pointer;
  PPSP_INTERFACE_DEVICE_DETAIL_DATA = ^PSP_INTERFACE_DEVICE_DETAIL_DATA;
  PSP_INTERFACE_DEVICE_DETAIL_DATA = ^SP_INTERFACE_DEVICE_DETAIL_DATA;
  SP_INTERFACE_DEVICE_DETAIL_DATA=packed record
    cbSize: cardinal;
    DevicePath: array[0..1019]of char; // this does not quite match the declaration in the ddk (which defines a zero length array)
  end;
  PSP_DEVICE_INTERFACE_DETAIL_DATA = PSP_INTERFACE_DEVICE_DETAIL_DATA;
  SP_DEVICE_INTERFACE_DETAIL_DATA = SP_INTERFACE_DEVICE_DETAIL_DATA;

  PSPDeviceInterfaceData = ^TSPDeviceInterfaceData;
  SP_DEVICE_INTERFACE_DATA = packed record
    cbSize: cardinal;
    InterfaceClassGuid: TGUID;
    Flags: cardinal;
    Reserved: Pointer;
  end;
  TSPDeviceInterfaceData = SP_DEVICE_INTERFACE_DATA;

  PHIDD_Attributes = ^THIDDAttributes;
  HIDD_ATTRIBUTES = record
    Size:          cardinal; // size of structure (set before call)
    VendorID:      Word;
    ProductID:     Word;
    VersionNumber: Word;
    //
    // Additional fields will be added to the end of this structure.
    //
  end;
  THIDDAttributes = HIDD_ATTRIBUTES;
const
  setupapi = 'setupapi.dll';
  hid = 'hid.dll';
  procedure HidD_GetHidGuid(var HidGuid: TGUID);stdcall; external hid;
  function SetupDiGetClassDevsA(const ClassGuid: TGUID; Enumerator: PChar;hwndParent: HWND; Flags: cardinal): THandle; stdcall; external setupapi;
  function SetupDiEnumDeviceInterfaces(DeviceInfoSet: THandle; DeviceInfoData: PSP_DEVINFO_DATA; var InterfaceClassGuid: TGuid; MemberIndex: cardinal; DeviceInterfaceData: PSP_INTERFACE_DEVICE_DETAIL_DATA): boolean; stdcall; external setupapi;
  function SetupDiGetDeviceInterfaceDetailA(DeviceInfoSet: THandle; DeviceInterfaceData: PSPDeviceInterfaceData; DeviceInterfaceDetailData: PSP_INTERFACE_DEVICE_DETAIL_DATA;
                                            DeviceInterfaceDetailDataSize: cardinal; var RequiredSize: cardinal; DeviceInfoData: PSP_DEVINFO_DATA):boolean;stdcall; external setupapi;
  function HidD_GetAttributes(HidDeviceObject: THandle; Attributes: PHIDD_ATTRIBUTES): boolean;stdcall; external hid;
  function HidD_GetPreparsedData(HidDeviceObject: THandle; var PreparsedData: Pointer): boolean;stdcall; external hid;
  function HidP_GetCaps(PreparsedData: Pointer; Capabilities: PHIDP_CAPS): integer; stdcall; external hid;
  function HidD_FreePreparsedData(PreparsedData: Pointer): boolean;stdcall; external hid;
  function SetupDiDestroyDeviceInfoList(DeviceInfoSet: THandle ): boolean;stdcall; external setupapi;



implementation

const
//stuff needed to communicate with the firmware
kVendorID  = $1781;
kProductID = $08C0;

//;command_type constants (from host to device)
CMD_SET_PORTS          =  00;   //; 2 params: data port 0 and 1
CMD_SET_PULLUPS        =  01;   //; 2 params: pullup register port 0 and 1
CMD_SET_REGISTER       =  02;   //; 2 params: register adr and value
CMD_SET_MEMORY         =  03;   //; 2 params: memory pointer and value
CMD_SET_REPORT_TYPE    =  04;   //; 2 params: report type and pointer
CMD_ENABLE_INTERRUPT   =  05;   //; 1 param: mask of interrupt (0x02 for microtimer)
CMD_DISABLE_INTERRUPT  =  06;   //; 1 param: mask of interrupt (0x02 for microtimer)

CMD_CLEAR_COMMANDLIST  = $07;   //; no params
CMD_CLEAR_MOTOR1       = $17;   //no params
CMD_CLEAR_MOTOR2       = $27;   //no params
CMD_CLEAR_MOTOR3       = $37;   //no params
CMD_CLEAR_MOTOR4       = $47;   //no params

CMD_DO_NOTHING         = $08;   //; for connection test
CMD_ADD_CMD1           = $10;   //; 3 params: (tick_L8)(tick_H6,D,S)[repeat cnt]
CMD_ADD_CMD2           = $20;   //; M2
CMD_ADD_CMD3           = $30;   //; M3
CMD_ADD_CMD4           = $40;   //; M4

CMD_START_RESET        = $09;   //; motormask,
CMD_SET_STATE          = $0A;   //; set state for port1 with mask

//;report_type constants  (device to host)
REPORT_NONE            =  0;
OBSOLETE_LASTCMDTICK   =  01;   //; no params
REPORT_PORTSTUFF       =  02;   //; no params
REPORT_VARIABLES       =  03;   //; param pointer into memory
REPORT_CURRENTTICK     =  04;   //; no params
REPORT_MOTOR_STATUS    =  05;   // out: port0,port1,m1.nextfree,m2.nextfree,m3.nextfree,m4.nextfree

CMDLISTSIZE            =  12;   // 14-2 byte for each motor

//cy7c63101 io ports
PORT0_Data             = $00;   // ; GPIO data port 0
PORT1_Data             = $01;   // ; GPIO data port 1
PORT0_Interrupt        = $04;   // ; Interrupt enable for port 0
PORT1_Interrupt        = $05;   // ; Interrupt enable for port 1
PORT0_Pullup           = $08;   // ; Pullup resistor control for port 0
PORT1_Pullup           = $09;   // ; Pullup resistor control for port 1

//device variable address
MotorResetFlags        = $12;



// USB stuff from windows
DBT_DEVICEARRIVAL     =  $8000;
DIGCF_PRESENT         = $02;
DIGCF_DEVICEINTERFACE = $10;


procedure DisplayLastError(S: string);
var lpMsgBuf: PChar; code: integer;
begin
  code:=GetLastError;
  if (code=0) or (code=997)or (code=259) then exit;
  FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER or FORMAT_MESSAGE_FROM_SYSTEM,nil,code,0,@lpMsgBuf,0,nil);
  if lpMsgBuf<>nil then begin
    MessageBox(0,lpMsgBuf,'GetLastError',MB_OK);
    LocalFree(cardinal(lpMsgBuf));
  end else
    MessageBox(0,PChar(inttostr(code)),'GetLastError',MB_OK);
end;

function KemoUsb_FindDevice(VendorID: integer;ProductID: integer; PdetailData: PSP_INTERFACE_DEVICE_DETAIL_DATA; DeviceCaps: PHIDP_CAPS):boolean; stdcall;
var
  Capabilities: HIDP_CAPS;
  DeviceHandle: THandle;
  hDevInfo: THandle;
  HidGuid: TGUID;
  Length: cardinal;
  Required: cardinal;

  Attributes: HIDD_ATTRIBUTES;
  devInfoData: SP_DEVICE_INTERFACE_DATA;
  detailData: PSP_INTERFACE_DEVICE_DETAIL_DATA;
  PreparsedData: Pointer;//PHIDP_PREPARSED_DATA;
  MemberIndex: integer;
  MyDeviceDetected: boolean;
begin
  Length := 0;
  HidD_GetHidGuid(HidGuid);	// Get the GUID for all system HIDs.
  hDevInfo:=SetupDiGetClassDevsA(HidGuid,nil,0,	DIGCF_PRESENT or DIGCF_DEVICEINTERFACE);
  ZeroMemory(@devInfoData,sizeof(devInfoData));
  devInfoData.cbSize := sizeof(devInfoData);

  MyDeviceDetected:=false;
  MemberIndex:=0;
  while not MyDeviceDetected do begin

    Result:=SetupDiEnumDeviceInterfaces(hDevInfo,nil,HidGuid,MemberIndex,@devInfoData);

    if not Result then begin
      DisplayLastError('SetupDiEnumDeviceInterfaces');
      break; // no more devices
    end;

    //Get size (Length)
    Length := 0;
    SetupDiGetDeviceInterfaceDetailA(hDevInfo,@devInfoData,nil,0,Length,nil);
    inc(Length,4);

    GetMem(detailData,Length);
    detailData.cbSize := 5;//Length;//5;//Length; //sizeof( SP_DEVICE_INTERFACE_DETAIL_DATA); //windows does not expect actual size

    Required := 0;
    Result := SetupDiGetDeviceInterfaceDetailA(hDevInfo,@devInfoData,detailData,Length,Required,nil);

    if not Result then begin
      DisplayLastError('SetupDiGetDeviceInterfaceDetailA');
//      break; // no more devices
    end;
    DeviceHandle:=CreateFile   //Open a handle to the device.
      (detailData.DevicePath,
      GENERIC_READ or GENERIC_WRITE,
      FILE_SHARE_READ or FILE_SHARE_WRITE,
      nil,
      OPEN_EXISTING,
      0,
      0);
    if (DeviceHandle<>INVALID_HANDLE_VALUE) then begin
      Attributes.Size := sizeof(Attributes);
      Result := HidD_GetAttributes(DeviceHandle,@Attributes);
      if( not Result) then DisplayLastError('HidD_GetAttributes');
      if (Attributes.VendorID = VendorID)and (Attributes.ProductID = ProductID) then begin //Both the Product and Vendor IDs match.
        if(not HidD_GetPreparsedData(DeviceHandle,PreparsedData) ) then
          DisplayLastError('HidD_GetPreparsedData');
        if(0= HidP_GetCaps(PreparsedData,@Capabilities)) then
          DisplayLastError('HidP_GetCaps');
        if( HidD_FreePreparsedData(PreparsedData)) then
          DisplayLastError('HidD_FreePreparsedData') ;
        if (DeviceCaps<>nil) then
          CopyMemory(DeviceCaps,@Capabilities,sizeof(DeviceCaps^));
        if (PdetailData<>nil) then
          CopyMemory(@PdetailData.DevicePath[0],@detailData.DevicePath[0],PdetailData.cbSize-4);
        MyDeviceDetected := true;
      end;
      CloseHandle(DeviceHandle);
    end;
    FreeMem(detailData);
    inc(MemberIndex);
  end;
  if(not SetupDiDestroyDeviceInfoList(hDevInfo) ) then
    DisplayLastError('SetupDiDestroyDeviceInfoList');
  result := MyDeviceDetected;
end;

function TKemoUSB.Connect: boolean;
var
  DevicePathData: SP_INTERFACE_DEVICE_DETAIL_DATA;
  Report: array[0..15]of byte;
  n: cardinal;
  OS: TOverlapped;
  UsbDevicePath: PChar;
  len: integer;
begin

  if Connected then begin
    result := true;
    exit;
  end;

  result := false;

  if hWriteFileEvent<>0 then begin
    CloseHandle(hWriteFileEvent);
    hWriteFileEvent:=0;
  end;

  DevicePathData.cbSize := sizeof(SP_INTERFACE_DEVICE_DETAIL_DATA);
  if KemoUsb_FindDevice(kVendorID,kProductID,@DevicePathData,@UsbDeviceCaps) then begin
    result := true;
    len := StrLen(DevicePathData.DevicePath);
    GetMem(UsbDevicePath,len+1);
    StrCopy(UsbDevicePath,DevicePathData.DevicePath);
    hUSB := CreateFile(UsbDevicePath, GENERIC_READ or GENERIC_WRITE, FILE_SHARE_READ or FILE_SHARE_WRITE, nil, OPEN_EXISTING,FILE_FLAG_OVERLAPPED, 0);
    FreeMem(UsbDevicePath);
    hWriteFileEvent := CreateEvent(nil,true,false,nil);

    n:=0;
    while usbwHandleInUse and (n<1000) do begin
      Sleep(10);
      inc(n);
    end;
    usbwHandleInUse := true;
    with OS do begin
      Internal := 0;
      InternalHigh := 0;
      Offset := 0;
      OffsetHigh := 0;
      hEvent := hWriteFileEvent;
    end;
    Report[0] := 0; // report id, always 0;
    Report[1] := CMD_SET_PULLUPS; // Set pullup register
    Report[2] := $00; // port0
    Report[3] := $00; // port1
    Report[7] := $ff;
    Report[8] := $ff;
    WriteFile(hUSB,Report,UsbDeviceCaps.OutputReportByteLength,n,@OS);
    n := WaitForSingleObject(hWriteFileEvent,100);  // 100ms
    ResetEvent(hWriteFileEvent);
    if WAIT_OBJECT_0 <> n then begin
      if n=STATUS_TIMEOUT then begin
        CancelIo(hUSB);
        CloseHandle(hUSB);
        hUSB := 0;
        result := false;
      end;
    end;
    usbwHandleInUse := false;
  end;
end;

function TKemoUSB.Connected: boolean;
var
  Report: array[0..15]of byte;
  n: cardinal;
  OS: TOverlapped;
begin
  result := false;

  if hUSB=0 then begin
    exit;
  end;

  with OS do begin
    Internal := 0;
    InternalHigh := 0;
    Offset := 0;
    OffsetHigh := 0;
    hEvent := hWriteFileEvent;
  end;
  Report[0] := 0;
  Report[1] := CMD_DO_NOTHING;
  WriteFile(hUSB,Report,UsbDeviceCaps.OutputReportByteLength,n,@OS);
  n := WaitForSingleObject(hWriteFileEvent,200);  // 200ms
  ResetEvent(hWriteFileEvent);
  if WAIT_OBJECT_0 <> n then begin
    CloseHandle(hUSB);
    hUSB:=0;
  end else begin
    result := true;
  end;

end;

constructor TKemoUSB.Create;
begin
  hUSB := 0;
  hWriteFileEvent := 0;
end;

destructor TKemoUSB.Destroy;
begin
  Disconnect;
  inherited;
end;

procedure TKemoUSB.Disconnect;
begin
  if hUSB<>0 then begin
    CloseHandle(hUSB);
    hUSB := 0;
  end;
  if hWriteFileEvent<>0 then begin
    CloseHandle(hWriteFileEvent);
    hWriteFileEvent:=0;
  end;
end;

function TKemoUSB.SetState(state: byte): boolean;
begin
  result := WriteRaw(CMD_SET_PORTS,state,state);
end;

function TKemoUSB.WriteRaw(cmd,var1,var2: byte; var3: byte=$ff; var4: byte=$ff; var5: byte=$ff): boolean;
var
  n: cardinal;
  OS: TOverlapped;
  Report: array[0..15]of byte;
begin
  if hUSB=0 then begin
    result := false;
    exit;
  end;

  result := true;

  n:=0;
  while usbwHandleInUse and (n<1000) do begin
    Sleep(10);
    inc(n);
  end;
  usbwHandleInUse := true;

  with OS do begin
    Internal := 0;
    InternalHigh := 0;
    Offset := 0;
    OffsetHigh := 0;
    hEvent := hWriteFileEvent;
  end;

  DisplayLastError('GetState ReadFile()');
  Report[0] := 0;             // report id, always 0;
  Report[1] := cmd;
  Report[2] := var1;
  Report[3] := var2;
  Report[4] := var3;
  Report[5] := var4;
  Report[6] := var5;
  WriteFile(hUSB,Report[0],UsbDeviceCaps.OutputReportByteLength,n,@OS);
  n := WaitForSingleObject(hWriteFileEvent,100);  // 100ms
  DisplayLastError('GetState ReadFile()');                                 //997
  ResetEvent(hWriteFileEvent);
  if WAIT_OBJECT_0 <> n then begin
    if n=STATUS_TIMEOUT then begin
      if not CancelIo(hUSB) then begin
        Disconnect;
        Connect; // try a reconnect once
      end;
      WriteFile(hUSB,Report,UsbDeviceCaps.OutputReportByteLength,n,@OS);
      n := WaitForSingleObject(hWriteFileEvent,500);
      ResetEvent(hWriteFileEvent);
      if WAIT_OBJECT_0 <> n then begin
        Disconnect;
        result := false;
      end;
    end else begin
      Disconnect;
      result := false;
    end;
  end;
  usbwHandleInUse := false;
end;


end.
