#include "StdAfx.h"
#include "KemoUSB.h"

/*  This unit is an interface to the KEMO module M162 (and maybe other KEMO modules in the future.
  Version 1.00
  Author: Lazaros Penteridis, Kemo Electronic GmbH 2013

  everything you need is the class KemoUSB
 */

using namespace std;

KemoUSB::KemoUSB(void)
{
	hUSB = 0;
	hWriteFileEvent = 0;
}


KemoUSB::~KemoUSB(void)
{
	Disconnect();
}


bool KemoUSB::WriteRaw(unsigned char cmd, unsigned char var1, unsigned char var2, unsigned char var3, unsigned char var4, unsigned char var5)
{
	unsigned int n;
	OVERLAPPED OS;
	unsigned char Report[15];
	bool result;
	if (hUSB==0) {
		return false;
	}
	result=true;
	n=0;
	while (usbwHandleInUse && (n<1000)) {
		Sleep(10);
		n++;
	}
	usbwHandleInUse = true;

	OS.Internal = 0;
	OS.InternalHigh = 0;
	OS.Offset = 0;
	OS.OffsetHigh = 0;
	OS.hEvent = hWriteFileEvent;

	DisplayLastError("GetState ReadFile()");
	Report[0] = 0;
	Report[1] = cmd;
	Report[2] = var1;
	Report[3] = var2;
	Report[4] = var3;
	Report[5] = var4;
	Report[6] = var5;


	WriteFile(hUSB, Report, UsbDeviceCaps.OutputReportByteLength, NULL, &OS);
	n = WaitForSingleObject(hWriteFileEvent,100);  // 100ms
	DisplayLastError("GetState ReadFile()");
	ResetEvent(hWriteFileEvent);
	if (WAIT_OBJECT_0 != n) {
		if (n == STATUS_TIMEOUT) {
			if (!(CancelIo(hUSB))){
				Disconnect();
				Connect();
			}
			WriteFile(hUSB, Report, UsbDeviceCaps.OutputReportByteLength, NULL, &OS);
			n = WaitForSingleObject(hWriteFileEvent,500);
			ResetEvent(hWriteFileEvent);
			if (WAIT_OBJECT_0 != n) {
				Disconnect();
				result=false;
			}
		}
		else {
			Disconnect();
			result = false;
		}
	}
	usbwHandleInUse = false;
	return result;
}

bool KemoUSB::Connect(void)
{
	PSP_INTERFACE_DEVICE_DETAIL_DATA pDevicePathData;
	unsigned char Report[15];
	unsigned int n;
	OVERLAPPED OS;
	bool result;

	if (Connected()) return true;

	result = false;

	if (hWriteFileEvent) {
		CloseHandle(hWriteFileEvent);
		hWriteFileEvent = 0;
	}

	pDevicePathData = (PSP_INTERFACE_DEVICE_DETAIL_DATA) malloc(1000);

	pDevicePathData->cbSize = 5;
	if (KemoUsb_FindDevice(kVendorID, kProductID, pDevicePathData, &UsbDeviceCaps)) {
		result = true;

		hUSB = CreateFile(pDevicePathData->DevicePath, (GENERIC_READ | GENERIC_WRITE), (FILE_SHARE_READ | FILE_SHARE_WRITE), NULL, OPEN_EXISTING, FILE_FLAG_OVERLAPPED, 0);
		
		hWriteFileEvent = CreateEvent(NULL, true, false, NULL);

		n=0;
		while (usbwHandleInUse && (n<1000)) {
			Sleep(10);
			n++;
		}
		usbwHandleInUse = true;
		OS.Internal = 0;
		OS.InternalHigh = 0;
		OS.Offset = 0;
		OS.OffsetHigh = 0;
		OS.hEvent = hWriteFileEvent;

		Report[0] = 0; // report id, always 0;
		Report[1] = CMD_SET_PULLUPS; // Set pullup register
		Report[2] = 0x00;
		Report[3] = 0x00;
		Report[7] = 0xff;
		Report[8] = 0xff;
		WriteFile(hUSB, Report, UsbDeviceCaps.OutputReportByteLength, NULL, &OS);
		n = WaitForSingleObject(hWriteFileEvent,100);  // 100ms
		ResetEvent(hWriteFileEvent);
		if (WAIT_OBJECT_0 != n) {
			if (n == STATUS_TIMEOUT) {
				CancelIo(hUSB);
				CloseHandle(hUSB);
				hUSB = 0;
				result = false;
			}
		}
		usbwHandleInUse = false;
	}

	free(pDevicePathData);

	return result;
}


bool KemoUSB::Connected(void) 
{
	unsigned char Report[15];
	unsigned int n;
	OVERLAPPED OS;
	bool result = false;

	if (hUSB == 0) return result;
	
	OS.Internal = 0;
	OS.InternalHigh = 0;
	OS.Offset = 0;
	OS.OffsetHigh = 0;
	OS.hEvent = hWriteFileEvent;

	Report[0] = 0;
    Report[1] = CMD_DO_NOTHING;
	WriteFile(hUSB, Report, UsbDeviceCaps.OutputReportByteLength, NULL, &OS);
	n = WaitForSingleObject(hWriteFileEvent,200);  // 200ms
	ResetEvent(hWriteFileEvent);
	if (WAIT_OBJECT_0 != n) {
		CloseHandle(hUSB);
		hUSB = 0;
	}
	else result = true;
	return result;
}


void KemoUSB::Disconnect(void) 
{
	if (hUSB) {
		CloseHandle(hUSB);
		hUSB = 0;
	}

	if (hWriteFileEvent) {
		CloseHandle(hWriteFileEvent);
		hWriteFileEvent = 0;
	}
}

bool KemoUSB::SetState(unsigned char state)
{
	return WriteRaw(CMD_SET_PORTS, state, state);
}

BOOL KemoUsb_FindDevice(int VendorID, int ProductID, PSP_INTERFACE_DEVICE_DETAIL_DATA PdetailData, PHIDP_CAPS DeviceCaps) //check
{
	HIDP_CAPS Capabilities;
	HANDLE hDevInfo, DeviceHandle;
	GUID HidGuid;
	LPGUID PHidGuid;
//	int err;
	
	
	DWORD Length;
	PDWORD PLength, Required;
	BOOL result;
	PLength = &Length;

	HIDD_ATTRIBUTES Attributes;
	SP_DEVICE_INTERFACE_DATA devInfoData;
	PSP_INTERFACE_DEVICE_DETAIL_DATA detailData;
	PHIDP_PREPARSED_DATA PreparsedData;	
	int MemberIndex;
	bool MyDeviceDetected;

	PHidGuid = &HidGuid;
	Length = 0;
	HidD_GetHidGuid(PHidGuid);
	hDevInfo = SetupDiGetClassDevs(PHidGuid, NULL, 0, DIGCF_PRESENT | DIGCF_DEVICEINTERFACE);
	ZeroMemory(&devInfoData, sizeof(devInfoData));
	devInfoData.cbSize = sizeof(devInfoData);

	MyDeviceDetected = false;
	MemberIndex = 0;
	while (!MyDeviceDetected) {
		result = SetupDiEnumDeviceInterfaces(hDevInfo, NULL, PHidGuid, MemberIndex, &devInfoData);
		if (!result) {
			DisplayLastError("SetupDiEnumDeviceInterfaces");
			break;
		}

		Length = 0;
		SetupDiGetDeviceInterfaceDetail(hDevInfo, &devInfoData, NULL, 0, &Length, NULL);
		Length+=4;

		detailData = (PSP_INTERFACE_DEVICE_DETAIL_DATA) malloc(Length);
		detailData->cbSize = 5; //Length; //sizeof( SP_DEVICE_INTERFACE_DETAIL_DATA); //windows does not expect actual size

		Required = 0;
		result = SetupDiGetDeviceInterfaceDetail(hDevInfo, &devInfoData, detailData, Length, Required, NULL);


		DeviceHandle = CreateFile(detailData->DevicePath, GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING, 0, 0);
		
		if (DeviceHandle != INVALID_HANDLE_VALUE) {
			Attributes.Size = sizeof(Attributes);
			result = HidD_GetAttributes(DeviceHandle, &Attributes);
			if ((Attributes.VendorID == VendorID) && (Attributes.ProductID == ProductID)) { //Both the Product and Vendor IDs match.
				if (!HidD_GetPreparsedData(DeviceHandle, &PreparsedData)) DisplayLastError("HidD_GetPreparsedData");
				if (!HidP_GetCaps(PreparsedData, &Capabilities)) DisplayLastError("HidP_GetCaps");
				if (!HidD_FreePreparsedData(PreparsedData)) DisplayLastError("HidD_FreePreparsedData") ;
				
				if (DeviceCaps) {
					CopyMemory(DeviceCaps,&Capabilities,sizeof(*DeviceCaps));

				}
				
				if (PdetailData) {
					char *S = detailData->DevicePath;
					int len = strlen(S);
					char *T = &PdetailData->DevicePath[0];

					for (int i=0;i<len+1;i++) {						
						T[i] = S[i];
						if(!S[i]) break;
					}

				}

				MyDeviceDetected = true;
			}
			CloseHandle(DeviceHandle);
		}
		free(detailData);
		MemberIndex++;
	}
	if (!SetupDiDestroyDeviceInfoList(hDevInfo)) DisplayLastError("SetupDiDestroyDeviceInfoList");
	result = MyDeviceDetected;
	return result;
}